# -*- coding: utf-8 -*-

import json
import sys
import pygame
from pygame.locals import *
from pygame.gfxdraw import *
from copy import deepcopy
from math import sin,cos,pi
import urllib

config = {}
config['json_url'] = 'http://localhost:8080/dump1090/data.json'
config['only_callsign'] = True
config['only_csh_and_ces'] = True
config['display_width'] = 1024
config['display_height'] = 768
config['map_range'] = {'lat':[27.1902,35.1902],'lon':[116.3363,126.3363]}
config['use_proxy']=False
config['proxy']={'http': 'http://www.someproxy.com:3128'}

map_objs = {}
map_objs['ZSSS']={'type':'airport','lat':31.190278,'lon':121.336389,'vis':True}

flight_objs = {}

def read_config():
	global config 
	try:
		f = open('config.json','r')
		config = json.loads(f.read())
		f.close()
		return True
	except IOError:
		f = open('config.json','w')
		f.write(json.dumps(config,sort_keys=True,indent=2,separators=(',',':')))
		f.close()
		return False

def read_map_objs():
	global map_objs
	try:
		f = open('map_objs.json','r')
		map_objs = json.loads(f.read())
		f.close()
		return True
	except IOError:
		f = open('map_objs.json','w')
		f.write(json.dumps(map_objs,sort_keys=True,indent=2,separators=(',',':')))
		f.close()
		return False

def write_map_objs():
	global map_objs
	try:
		f = open('map_objs.json','w')
		f.write(json.dumps(map_objs,sort_keys=True,indent=2,separators=(',',':')))
		f.close()
		return True
	except IOError:
		return False

def read_flight_objs():
	global flight_objs
	pass

def check_map_range():
	global config
	global map_range
	h = map_range['lat'][1] - map_range['lat'][0]
	w = map_range['lon'][1] - map_range['lon'][0]
	print 'check_map_range: ',map_range,config['map_range']
	if map_range['lat'][0] < config['map_range']['lat'][0]:
		map_range['lat'][0] = deepcopy(config['map_range']['lat'][0])
	if map_range['lat'][1] > config['map_range']['lat'][1]:
		map_range['lat'][1] = deepcopy(config['map_range']['lat'][1])
	if map_range['lon'][0] < config['map_range']['lon'][0]:
		map_range['lon'][0] = deepcopy(config['map_range']['lon'][0])
	if map_range['lon'][1] > config['map_range']['lon'][1]:
		map_range['lon'][1] = deepcopy(config['map_range']['lon'][1])
	print 'check_map_range: ',map_range,config['map_range']
	return map_range

def show_top_msg(txt):
	global msg_fontobj
	global txtcolor
	global wndSurface
	msgObj = msg_fontobj.render(txt,False,txtcolor)
	msgRect = msgObj.get_rect()
	msgRect.top = 0
	msgRect.left = 10
	wndSurface.blit(msgObj,msgRect)
	return 1
def show_buttom_msg(txt):
	global msg_fontobj
	global txtcolor
	global wndSurface
	msgObj = msg_fontobj.render(txt,False,txtcolor)
	msgRect = msgObj.get_rect()
	msgRect.top = wndSurface.get_height()-msgObj.get_height()
	msgRect.left = 10
	wndSurface.blit(msgObj,msgRect)
	return 1

def draw_label(surface,obj,x,y):
	global label_fontobj
	global mapobjcolor
	global config
	#print 'draw_label: ',obj['label'],x,y
	labelObj = label_fontobj.render(obj['label'],False,mapobjcolor)
	labelRect = labelObj.get_rect()
	labelRect.top = y + config['symbol_size']
	labelRect.left = x - labelRect.width / 2
	surface.blit(labelObj,labelRect)
	return surface

def draw_apt(surface,obj,x,y):
	global mapobjcolor
	global config
	#print 'draw_apt: ',obj['label'],x,y
	aacircle(surface,int(x),int(y),config['symbol_size'],mapobjcolor)
	line(surface, int(x+config['symbol_size']),int(y), int(x-config['symbol_size']) , int(y), mapobjcolor)
	line(surface, int(x),int(y+config['symbol_size']), int(x) , int(y-config['symbol_size']), mapobjcolor)
	surface = draw_label(surface,obj,x,y)
	return surface

def draw_waypt(surface,obj,x,y):
	global mapobjcolor
	global config
	#print 'draw_waypt: ',obj['label'],x,y
	dx = abs(config['symbol_size']*cos(0.5235987755982989))
	dy = abs(config['symbol_size']/2)
	aatrigon(surface,int(x),int(y-config['symbol_size']),
			int(x-dx),int(y+dy),
			int(x+dx),int(y+dy),mapobjcolor)
	surface = draw_label(surface,obj,x,y)
	return surface

def draw_point(surface,obj,x,y):
	global mapobjcolor
	global config
	#print 'draw_point: ',obj['label'],x,y
	d = config['symbol_size']*1.414/2
	line(surface,int(x-d),int(y-d),int(x+d),int(y+d),mapobjcolor)
	line(surface,int(x+d),int(y-d),int(x-d),int(y+d),mapobjcolor)
	surface = draw_label(surface,obj,x,y)
	return surface

def draw_map(surface,lat,lon):
	global config
	global map_objs
	
	w = lon[1] - lon[0]
	h = lat[1] - lat[0]
	for key in map_objs.keys():
		mo = map_objs[key]
		if (mo['lat'] > lat[0] and mo['lat'] < lat[1] 
		and mo['lon'] > lon[0] and mo['lon'] < lon[1]
		and mo['vis'] == True):
			x = surface.get_width()*(mo['lon'] - lon[0])/w
			y = surface.get_height()*(lat[1] - mo['lat'])/h
			if mo['type'] == 'APT':
				surface = draw_apt(surface,mo,x,y)
			elif mo['type'] == 'VORD' or mo['type'] == 'WAYPT':
				surface = draw_waypt(surface,mo,x,y)
			elif mo['type'] == 'POINT':
				surface = draw_point(surface,mo,x,y)
	return surface

def read_flight_objs():
	global config
	global flight_objs
	if config['use_proxy'] == False:
		config['proxy'] = {}
	try:
		f = urllib.urlopen(config['json_url'], proxies=config['proxy'])
		flight_objs = json.loads(f.read())
		f.close()
		return True
	except:
		return False

def filter_flight_objs():
	global config
	global flight_objs
	for fo in flight_objs:
		fo['flight'] = fo['flight'].strip()
		if fo['lat'] == 0.0 or fo['lon'] == 0.0:
			fo['vis'] = False
			continue
		else:
			fo['vis'] = True
		if config['only_callsign'] == True:
			if len(fo['flight']) == 0:
				fo['vis'] = False
				continue
		if len(fo['flight']) > 0 and config['only_csh_and_ces'] == True:
			if fo['flight'][:3] != 'CSH' or fo['flight'][:3] != 'CES':
				fo['vis']=False

def draw_flight_label(surface,obj,x,y):
	global label_fontobj
	global flightobjcolor
	global config
	#print 'draw_label: ',obj['hex'],x,y
	if len(obj['flight'])==0:
		label = '#'+obj['hex']+'/'+str(obj['altitude'])
	else:
		label = obj['flight']+'/'+str(obj['altitude'])
	labelObj = label_fontobj.render(label,False,flightobjcolor)
	labelRect = labelObj.get_rect()
	h = labelRect.height
	labelRect.top = y + config['symbol_size']
	labelRect.left = x - labelRect.width / 2
	surface.blit(labelObj,labelRect)
	top = labelRect.top + h
	left = labelRect.left

	label = ''
	if obj['squawk']!='0000':
		label = obj['squawk']
	if obj['speed']!=0:
		label += ('/'+str(obj['speed']))
	labelObj = label_fontobj.render(label,False,flightobjcolor)
	labelRect = labelObj.get_rect()
	labelRect.top = top
	labelRect.left = left
	surface.blit(labelObj,labelRect)
	return surface

def draw_flight(surface,obj,x,y):
	global flightobjcolor
	global config
	#print 'draw_flight: ',obj['hex'],x,y
	aacircle(surface,int(x),int(y),config['symbol_size'],flightobjcolor)
	surface = draw_flight_label(surface,obj,x,y)
	if obj['track'] != 0:
		x2 = x+1.5*config['symbol_size']*cos((90-obj['track'])*2*pi/360)
		y2 = y-1.5*config['symbol_size']*sin((90-obj['track'])*2*pi/360)
		line(surface, int(x),int(y), int(x2) , int(y2), flightobjcolor)
	return surface

def draw_flight_objs(surface,lat,lon):
	global config
	global flight_objs
	
	w = lon[1] - lon[0]
	h = lat[1] - lat[0]
	for fo in flight_objs:
		if (fo['lat'] > lat[0] and fo['lat'] < lat[1] 
		and fo['lon'] > lon[0] and fo['lon'] < lon[1]
		and fo['vis'] == True):
			x = surface.get_width()*(fo['lon'] - lon[0])/w
			y = surface.get_height()*(lat[1] - fo['lat'])/h
			surface = draw_flight(surface,fo,x,y)
	return surface

read_config()
read_map_objs()

pygame.init()
clock = pygame.time.Clock()
wndSurface = pygame.display.set_mode((config['display_width'],config['display_height']))
pygame.display.set_caption('py_radar')

bgcolor = pygame.Color(0,32,85)
txtcolor = pygame.Color(255,255,255)
mapobjcolor = pygame.Color(230,230,230)
flightobjcolor = pygame.Color(255,227,21)
msg_fontobj = pygame.font.Font('freesansbold.ttf',25)
label_fontobj = pygame.font.Font('freesansbold.ttf',15)


map_range = deepcopy(config['map_range'])
draw_map(wndSurface,map_range['lat'],map_range['lon'])

def redraw():
	global wndSurface
	global map_range
	wndSurface.fill(bgcolor)
	wndSurface = draw_map(wndSurface,map_range['lat'],map_range['lon'])
	wndSurface = draw_flight_objs(wndSurface,map_range['lat'],map_range['lon'])

while(1):
	if read_flight_objs():
		filter_flight_objs()
		pygame.event.post(pygame.event.Event(USEREVENT))
	for event in pygame.event.get():
		if event.type == QUIT:
			pygame.quit()
			write_map_objs()
			sys.exit()
		elif event.type == KEYUP:
			if event.key == K_EQUALS:
				dw = (1-config['zoom'])*(map_range['lon'][1] - map_range['lon'][0])
				dh = (1-config['zoom'])*(map_range['lat'][1] - map_range['lat'][0])
				map_range['lon'][0] += dw
				map_range['lon'][1] -= dw
				map_range['lat'][0] += dh
				map_range['lat'][1] -= dh
				check_map_range()
				redraw()
			elif event.key == K_MINUS:
				dw = (1-config['zoom'])*(map_range['lon'][1] - map_range['lon'][0])
				dh = (1-config['zoom'])*(map_range['lat'][1] - map_range['lat'][0])
				map_range['lon'][0] -= dw
				map_range['lon'][1] += dw
				map_range['lat'][0] -= dh
				map_range['lat'][1] += dh
				check_map_range()
				redraw()
			elif event.key == K_w or event.key == K_UP:
				dh = (1-config['zoom'])*(map_range['lat'][1] - map_range['lat'][0])
				if map_range['lat'][1] + dh <= config['map_range']['lat'][1]:
					map_range['lat'][0] += dh
					map_range['lat'][1] += dh
				redraw()
			elif event.key == K_s or event.key == K_DOWN:
				dh = (1-config['zoom'])*(map_range['lat'][1] - map_range['lat'][0])
				if map_range['lat'][0] - dh >= config['map_range']['lat'][0]:
					map_range['lat'][0] -= dh
					map_range['lat'][1] -= dh
				redraw()
			elif event.key == K_a or event.key == K_LEFT:
				dw = (1-config['zoom'])*(map_range['lon'][1] - map_range['lon'][0])
				if map_range['lon'][0] - dh >= config['map_range']['lon'][0]:
					map_range['lon'][0] -= dw
					map_range['lon'][1] -= dw
				redraw()
			elif event.key == K_d or event.key == K_RIGHT:
				dw = (1-config['zoom'])*(map_range['lon'][1] - map_range['lon'][0])
				if map_range['lon'][1] + dh <= config['map_range']['lon'][1]:
					map_range['lon'][0] += dw
					map_range['lon'][1] += dw
				redraw()
			elif event.key == K_SPACE or event.key == K_ESCAPE:
				map_range = deepcopy(config['map_range'])
				redraw()
		elif event.type == USEREVENT:
			redraw()
	pygame.display.update()
	clock.tick(30)

